"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRouteUrl = exports.mountControllers = exports.getMountPoints = exports.getControllers = exports.All = exports.Head = exports.Delete = exports.Patch = exports.Put = exports.Post = exports.Get = exports.Named = exports.After = exports.Before = exports.Controller = void 0;
const koa_router_1 = __importDefault(require("koa-router"));
require("reflect-metadata");
const CONTROLLER_KEY = Symbol('avanzu.kernel.controller');
const MIDDLEWARE_KEY = Symbol('avanzu.kernel.middleware');
const BEFORE_MIDDLEWARE_KEY = Symbol('avanzu.kernel.middleware.before');
const AFTER_MIDDLEWARE_KEY = Symbol('avanzu.kernel.middleware.after');
const ROUTE_KEY = Symbol('avanzu.kernel.route');
const HTTP_VERB_KEY = Symbol('avanzu.kernel.httpVerb');
const ROUTE_NAME = Symbol('avanzu.kernel.routename');
const controllerMap = new Set();
const urlGenerator = new koa_router_1.default();
function Controller(prefix = '', ...middlewares) {
    return function controllerDecorator(target) {
        Reflect.defineMetadata(CONTROLLER_KEY, prefix, target);
        Reflect.defineMetadata(MIDDLEWARE_KEY, middlewares, target);
        controllerMap.add(target);
    };
}
exports.Controller = Controller;
function Route(method, route, middlewares) {
    return function routeDecorator(target, propertyKey) {
        Reflect.defineMetadata(HTTP_VERB_KEY, method, target, propertyKey);
        Reflect.defineMetadata(ROUTE_KEY, route, target, propertyKey);
        Reflect.defineMetadata(MIDDLEWARE_KEY, middlewares, target, propertyKey);
    };
}
function Before(...middlewares) {
    return function beforeMiddlewareDecorator(target, propertyKey) {
        Reflect.defineMetadata(BEFORE_MIDDLEWARE_KEY, middlewares, target, propertyKey);
    };
}
exports.Before = Before;
function After(...middlewares) {
    return function beforeMiddlewareDecorator(target, propertyKey) {
        Reflect.defineMetadata(AFTER_MIDDLEWARE_KEY, middlewares, target, propertyKey);
    };
}
exports.After = After;
function Named(name) {
    return function (target, propertyKey) {
        Reflect.defineMetadata(ROUTE_NAME, name, target, propertyKey);
    };
}
exports.Named = Named;
function Get(route = '/', ...middlewares) {
    return Route('get', route, middlewares);
}
exports.Get = Get;
function Post(route = '/', ...middlewares) {
    return Route('post', route, middlewares);
}
exports.Post = Post;
function Put(route = '/', ...middlewares) {
    return Route('put', route, middlewares);
}
exports.Put = Put;
function Patch(route = '/', ...middlewares) {
    return Route('patch', route, middlewares);
}
exports.Patch = Patch;
function Delete(route = '/', ...middlewares) {
    return Route('delete', route, middlewares);
}
exports.Delete = Delete;
function Head(route = '/', ...middlewares) {
    return Route('head', route, middlewares);
}
exports.Head = Head;
function All(route = '/', ...middlewares) {
    return Route('all', route, middlewares);
}
exports.All = All;
/**
 * @internal
 */
function getControllers() {
    return Array.from(controllerMap);
}
exports.getControllers = getControllers;
/**
 * @internal
 */
function getMountPoints(target) {
    const proto = target.prototype;
    const ctor = proto.constructor;
    const mount = { endpoints: [], middlewares: [] };
    if (Reflect.hasMetadata(CONTROLLER_KEY, ctor)) {
        mount.prefix = Reflect.getMetadata(CONTROLLER_KEY, ctor);
    }
    if (Reflect.hasMetadata(MIDDLEWARE_KEY, ctor)) {
        mount.middlewares = Reflect.getMetadata(MIDDLEWARE_KEY, ctor);
    }
    for (const method of Object.getOwnPropertyNames(proto)) {
        if (false === Reflect.hasMetadata(ROUTE_KEY, proto, method)) {
            continue;
        }
        const route = Reflect.getMetadata(ROUTE_KEY, proto, method);
        const endpoint = {
            target,
            route,
            verb: 'all',
            method,
            middlewares: [],
            before: [],
            after: [],
        };
        if (Reflect.hasMetadata(HTTP_VERB_KEY, proto, method)) {
            endpoint.verb = Reflect.getMetadata(HTTP_VERB_KEY, proto, method);
        }
        if (Reflect.hasMetadata(MIDDLEWARE_KEY, proto, method)) {
            endpoint.middlewares = Reflect.getMetadata(MIDDLEWARE_KEY, proto, method);
        }
        if (Reflect.hasMetadata(BEFORE_MIDDLEWARE_KEY, proto, method)) {
            endpoint.before = Reflect.getMetadata(BEFORE_MIDDLEWARE_KEY, proto, method);
        }
        if (Reflect.hasMetadata(AFTER_MIDDLEWARE_KEY, proto, method)) {
            endpoint.after = Reflect.getMetadata(AFTER_MIDDLEWARE_KEY, proto, method);
        }
        if (Reflect.hasMetadata(ROUTE_NAME, proto, method)) {
            endpoint.name = Reflect.getMetadata(ROUTE_NAME, proto, method);
        }
        mount.endpoints.push(endpoint);
    }
    return mount;
}
exports.getMountPoints = getMountPoints;
/**
 * @internal
 */
function mountControllers(globalPrefix) {
    const root = new koa_router_1.default({ prefix: globalPrefix });
    for (const controller of getControllers()) {
        const mountpoints = getMountPoints(controller);
        const router = new koa_router_1.default({ prefix: mountpoints.prefix });
        for (const middleware of mountpoints.middlewares) {
            router.use(middleware);
        }
        for (const endpoint of mountpoints.endpoints) {
            const handler = createHandler(endpoint);
            const middlewares = [
                ...endpoint.before,
                ...endpoint.middlewares,
                handler,
                ...endpoint.after,
            ];
            router[endpoint.verb](endpoint.route, ...middlewares);
            if (endpoint.name) {
                const fullPath = getFullPath(globalPrefix, mountpoints, endpoint);
                urlGenerator.register(fullPath, [endpoint.verb], [], { name: endpoint.name });
            }
        }
        root.use(router.routes());
    }
    return root;
}
exports.mountControllers = mountControllers;
function getFullPath(globalPrefix, mountpoints, endpoint) {
    return [globalPrefix, mountpoints.prefix, endpoint.route]
        .map(s => s?.replace(/(^\/|\/$)/g, ''))
        .filter(Boolean)
        .join('/');
}
function getRouteUrl(name, params, options) {
    return urlGenerator.url(name, params, options);
}
exports.getRouteUrl = getRouteUrl;
function createHandler(endpoint) {
    return async function dispatchRequest(context, next) {
        const instance = context.scope.build(endpoint.target);
        await instance[endpoint.method](context, next);
    };
}
//# sourceMappingURL=routing.js.map