"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getResolver = exports.getUseCase = exports.getUseCases = exports.InputResolver = exports.UseCase = exports.USECASE_TAG = void 0;
require("reflect-metadata");
const util_1 = require("./util");
const USECASE_KEY = Symbol('avanzu.kernel.useCase');
const USECASE_INFO = Symbol('avanzu.kernel.useCase:info');
exports.USECASE_TAG = 'UseCase';
const useCases = new Map();
const resolvers = new Map();
function UseCase(options) {
    return function useCaseDecorator(target) {
        if (false === (0, util_1.isClass)(target))
            return;
        Reflect.defineMetadata(USECASE_KEY, true, target);
        Reflect.defineMetadata(USECASE_INFO, options, target);
        useCases.set(options.id, target);
    };
}
exports.UseCase = UseCase;
function InputResolver(targetUseCase) {
    return function payloadResolverDecorator(target) {
        resolvers.set(targetUseCase, target);
    };
}
exports.InputResolver = InputResolver;
function getUseCases() {
    const entries = [];
    for (const [, useCase] of useCases) {
        const info = createInfo(useCase);
        entries.push(info);
    }
    return entries;
}
exports.getUseCases = getUseCases;
function createInfo(useCase) {
    const proto = useCase.prototype;
    const ctor = proto.constructor;
    const info = Reflect.getMetadata(USECASE_INFO, ctor);
    return { ...info, useCase, tags: [exports.USECASE_TAG] };
}
function getUseCase(id) {
    if (useCases.has(id)) {
        const useCase = useCases.get(id);
        return createInfo(useCase);
    }
    return null;
}
exports.getUseCase = getUseCase;
function getResolver(useCase) {
    if (resolvers.has(useCase)) {
        return resolvers.get(useCase);
    }
    return null;
}
exports.getResolver = getResolver;
//# sourceMappingURL=useCase.js.map