/// <reference types="node" />
import { AddressInfo, Server } from 'net';
import * as Types from './interfaces';
/**
 * The kernel class is the orchestration layer of your application.
 * It serves as the entry point that initializes the container, prepares the Koa application, wires up middleware, and loads your controller routes.
 *
 */
export declare abstract class Kernel<Config extends Types.Configuration<Types.ConfigOptions>, App extends Types.App<any, any, any>, Container extends Types.Container> {
    protected app: App;
    protected container: Container;
    protected logger: Types.Logger;
    protected options: Config;
    protected _server?: Server;
    protected _address?: AddressInfo;
    constructor(options: Config);
    get server(): Server;
    get address(): AddressInfo;
    boot(): Promise<void>;
    serve(): Promise<Server>;
    shutdown(): Promise<void>;
    protected createContainer(): Container;
    protected buildContainer(): Promise<void>;
    protected addMiddlewares(): void;
    protected createApp(): App;
    protected abstract createContainerBuilder(): Types.ContainerBuilder<Container>;
    protected middlewares(): Types.AppMiddleware<any, any, any>[];
    protected abstract createLogger(): Types.Logger;
    protected loadControllers(): void;
}
