"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenApi = void 0;
const oas_builder_1 = __importDefault(require("@avanzu/oas-builder"));
const controllerDocumentor_1 = require("./controllerDocumentor");
const decorators_1 = require("../../decorators");
class OpenApi {
    constructor(info, container) {
        this.info = info;
        this.container = container;
    }
    generate() {
        this.paths = new Map();
        this.document = oas_builder_1.default.document();
        this.tags = new Set();
        const info = oas_builder_1.default.info()
            .title(this.info.title)
            .summary(this.info.description)
            .version(this.info.version);
        for (const controller of (0, decorators_1.getControllers)()) {
            this.processController(controller);
        }
        this.document = this.document.paths(Object.fromEntries(this.paths));
        return this.document.info(info).valueOf();
    }
    addTag(opts) {
        if (null == opts.tag)
            return;
        if (!this.tags.has(opts.tag.name)) {
            this.document = this.document.tag(oas_builder_1.default.tag().name(opts.tag.name).description(opts.tag.description));
            this.tags.add(opts.tag.name);
        }
    }
    processController(controller) {
        const opts = (0, decorators_1.getApiDocs)(controller);
        if (null == opts)
            return;
        this.addTag(opts);
        const generator = opts.generator ? opts.generator() : controllerDocumentor_1.ControllerDocumentor;
        const mountpoint = (0, decorators_1.getMountPoints)(controller);
        const documentor = this.container.build(generator);
        for (const endpoint of mountpoint.endpoints) {
            const context = {
                endpoint,
                mountpoint,
                opts,
                info: this.info,
                container: this.container,
            };
            if (this.isCustomDocumentor(documentor)) {
                this.useCustomDocumentor(endpoint, documentor, context);
            }
            else if (this.isGenericDocumentor(documentor)) {
                this.useGenericDocumentor(documentor, context);
            }
        }
    }
    isGenericDocumentor(value) {
        return 'kind' in value && value.kind === 'generic';
    }
    isCustomDocumentor(value) {
        return 'kind' in value && value.kind === 'custom';
    }
    useCustomDocumentor(endpoint, documentor, context) {
        if (false === endpoint.method in documentor) {
            return;
        }
        const { route, path } = documentor[String(endpoint.method)].call(documentor, context);
        this.paths.set(route, path);
    }
    useGenericDocumentor(documentor, context) {
        const { route, path } = documentor.getInfo(context);
        this.paths.set(route, path);
    }
}
exports.OpenApi = OpenApi;
//# sourceMappingURL=openapi.js.map