"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemotePlugin = exports.PluginResponseSchema = void 0;
// import axios from 'axios'
const http_client_1 = require("@avanzu/http-client");
const crypto = __importStar(require("crypto"));
const typebox_1 = require("@sinclair/typebox");
const errors_1 = require("../../errors");
exports.PluginResponseSchema = typebox_1.Type.Object({
    hook: typebox_1.Type.String(),
    data: typebox_1.Type.Record(typebox_1.Type.String(), typebox_1.Type.Unknown()),
    signature: typebox_1.Type.String()
});
class RemotePlugin {
    constructor(definition, validator) {
        this.definition = definition;
        this.validator = validator;
    }
    name() {
        return this.definition.name;
    }
    version() {
        return this.definition.version;
    }
    dependsOn() {
        return this.definition.dependsOn;
    }
    description() {
        throw new Error('Method not implemented.');
    }
    async invoke(context) {
        const endpoint = this.definition.hooks[context.hook];
        if (false === Boolean(endpoint)) {
            return context;
        }
        try {
            const client = new http_client_1.HttpClient({ baseUrl: this.definition.baseURL });
            // const url = new URL(endpoint, this.definition.baseURL)
            const serializedContext = context.toJSON();
            const response = await client.post(endpoint, serializedContext);
            const validation = await this.validator.validate(exports.PluginResponseSchema, response.data);
            if (false === validation.isValid) {
                throw new errors_1.ValidationError(validation.errors);
            }
            const { signature, ...data } = response;
            if (false === this.isSignatureValid(data, signature)) {
                throw new Error('Invalid signature; data may have been tampered with.');
            }
            context.applyDTO(response);
            return context;
        }
        catch (error) {
            throw new Error(`Failed to invoke remote plugin ${this.name}: ${error}`);
        }
    }
    isSignatureValid(data, signature) {
        const verify = crypto.createVerify('SHA256');
        verify.update(JSON.stringify(data));
        return verify.verify(this.definition.publicKey, signature, 'hex'); // Verify the signature
    }
}
exports.RemotePlugin = RemotePlugin;
//# sourceMappingURL=RemotePlugin.js.map