"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginInvocationError = exports.MissingDependency = exports.CircularDependency = exports.PluginAlreadyRegistered = void 0;
const http_status_codes_1 = require("http-status-codes");
const errors_1 = require("../../errors");
class PluginAlreadyRegistered extends errors_1.KernelError {
    constructor(plugin) {
        super(`Plugin ${plugin.name()} is already registered.`);
        this.plugin = plugin;
        this.status = http_status_codes_1.StatusCodes.INTERNAL_SERVER_ERROR;
        this.reason = http_status_codes_1.ReasonPhrases.INTERNAL_SERVER_ERROR;
        this.code = errors_1.ErrorCodes.KERNEL;
    }
}
exports.PluginAlreadyRegistered = PluginAlreadyRegistered;
class CircularDependency extends errors_1.KernelError {
    constructor(plugin, visiting) {
        super(`Plugin ${plugin.name()} has a circular dependency.`);
        this.plugin = plugin;
        this.visiting = visiting;
        this.status = http_status_codes_1.StatusCodes.INTERNAL_SERVER_ERROR;
        this.reason = http_status_codes_1.ReasonPhrases.INTERNAL_SERVER_ERROR;
        this.code = errors_1.ErrorCodes.KERNEL;
    }
    get details() {
        return {
            plugin: this.plugin.name(),
            visiting: this.visiting,
        };
    }
}
exports.CircularDependency = CircularDependency;
class MissingDependency extends errors_1.KernelError {
    constructor(plugin, dependency) {
        super(`Plugin ${plugin.name()} depends on ${dependency} which is not provided.`);
        this.plugin = plugin;
        this.dependency = dependency;
        this.status = http_status_codes_1.StatusCodes.INTERNAL_SERVER_ERROR;
        this.reason = http_status_codes_1.ReasonPhrases.INTERNAL_SERVER_ERROR;
        this.code = errors_1.ErrorCodes.KERNEL;
    }
}
exports.MissingDependency = MissingDependency;
class PluginInvocationError extends errors_1.KernelError {
    constructor(plugin, error) {
        super(`Plugin ${plugin.name()} failed to execute.`);
        this.plugin = plugin;
        this.error = error;
        this.status = http_status_codes_1.StatusCodes.INTERNAL_SERVER_ERROR;
        this.reason = http_status_codes_1.ReasonPhrases.INTERNAL_SERVER_ERROR;
        this.code = errors_1.ErrorCodes.KERNEL;
    }
    get details() {
        return {
            plugin: this.plugin.name(),
            error: String(this.error),
        };
    }
}
exports.PluginInvocationError = PluginInvocationError;
//# sourceMappingURL=errors.js.map