import { PluginHookContext } from './HookContext';
export type PluginDefinition<Config extends {} = {}> = LocalPluginDefinition<Config> | PackagePluginDefinition<Config> | RemotePluginDefinition<Config>;
export type LocalPluginDefinition<Config extends {} = {}> = {
    modulePath: string;
    className: string;
    config: Config;
    type: 'local';
};
export type PackagePluginDefinition<Config extends {} = {}> = {
    packageName: string;
    className: string;
    config: Config;
    type: 'package';
};
export type RemotePluginDefinition<Config extends {} = {}> = {
    baseURL: string;
    hooks: Record<string, string>;
    name: string;
    version: string;
    config: Config;
    type: 'remote';
    dependsOn: string[];
    publicKey: string;
};
export interface Plugin {
    name(): string;
    version(): string;
    dependsOn(): string[];
    description(): string;
    invoke(context: PluginHookContext): Promise<PluginHookContext>;
}
export interface Initializable {
    initialize(): Promise<void>;
}
export declare function isInitializable(value: unknown): value is Initializable;
export interface Disposable {
    dispose(): Promise<void>;
}
export declare function isDisposable(value: unknown): value is Disposable;
