"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginManager = void 0;
const Errors = __importStar(require("./errors"));
const errors_1 = require("./errors");
const plugin_1 = require("./plugin");
class PluginManager {
    constructor(pluginRegistry) {
        this.pluginRegistry = pluginRegistry;
        this.plugins = new Map();
    }
    async initialize() {
        const plugins = await this.pluginRegistry.loadPluginsEnabled();
        this.register(...plugins);
    }
    async dispose() {
        for (let [, plugin] of this.plugins) {
            if ((0, plugin_1.isDisposable)(plugin))
                await plugin.dispose();
        }
        this.plugins.clear();
    }
    register(...plugins) {
        plugins = this.sort(plugins);
        for (let plugin of plugins) {
            if (this.plugins.has(plugin.name())) {
                throw new Errors.PluginAlreadyRegistered(plugin);
            }
            this.plugins.set(plugin.name(), plugin);
        }
    }
    async invoke(context) {
        for (let [name, plugin] of this.plugins) {
            try {
                context = await plugin.invoke(context);
            }
            catch (error) {
                throw new errors_1.PluginInvocationError(plugin, error);
            }
        }
        return context;
    }
    sort(plugins) {
        const visited = new Map();
        const stack = [];
        const visiting = new Set();
        const pool = new Map(plugins.map((plugin) => [plugin.name(), plugin]));
        function visit(plugin) {
            if (visiting.has(plugin.name())) {
                throw new Errors.CircularDependency(plugin, Array.from(visiting).concat(plugin.name()));
            }
            if (true === visited.get(plugin.name())) {
                return;
            }
            visiting.add(plugin.name());
            for (const dependency of plugin.dependsOn()) {
                if (false === pool.has(dependency)) {
                    throw new Errors.MissingDependency(plugin, dependency);
                }
                visit(pool.get(dependency));
            }
            visiting.delete(plugin.name());
            visited.set(plugin.name(), true);
            stack.push(plugin);
        }
        for (const plugin of plugins) {
            if (visited.get(plugin.name())) {
                continue;
            }
            visit(plugin);
        }
        return stack.reverse();
    }
}
exports.PluginManager = PluginManager;
//# sourceMappingURL=pluginManager.js.map