"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginRegistry = void 0;
const Path = __importStar(require("node:path"));
const plugin_1 = require("./plugin");
const RemotePlugin_1 = require("./RemotePlugin");
class PluginRegistry {
    constructor(pluginRepository, validator, baseDir) {
        this.pluginRepository = pluginRepository;
        this.validator = validator;
        if (false === Boolean(baseDir)) {
            baseDir = Path.resolve(process.cwd(), 'plugins');
        }
        this.baseDir = baseDir;
    }
    async loadPluginsEnabled() {
        const enabledPlugins = await this.pluginRepository.getPluginsEnabled();
        return this.loadPlugins(enabledPlugins);
    }
    /**
     * Load plugins from various sources
     */
    async loadPlugins(pluginDefinitions) {
        const plugins = [];
        for (const definition of pluginDefinitions) {
            const plugin = await this.create(definition);
            if ((0, plugin_1.isInitializable)(plugin))
                await plugin.initialize();
            plugins.push(plugin);
        }
        return plugins;
    }
    resolveModulePath(definition) {
        if (definition.type === 'local')
            return Path.resolve(this.baseDir, definition.modulePath);
        if (definition.type === 'package')
            return definition.packageName;
    }
    /**
     * Instantiate plugin using the provided definition.
     * This could dynamically import a module, or use a known plugin class.
     */
    async create(definition) {
        if (definition.type === 'remote')
            return new RemotePlugin_1.RemotePlugin(definition, this.validator);
        const modulePath = this.resolveModulePath(definition); // Path to the plugin module (e.g., './plugins/myPlugin')
        const className = definition.className; // Name of the exported plugin class (e.g., 'MyPlugin')
        // Dynamically import the module
        const module = await import(modulePath);
        if (!module[className]) {
            throw new Error(`Class ${className} not found in module ${modulePath}`);
        }
        // Instantiate the plugin class
        const PluginClass = module[className];
        // You can also pass configuration or context into the constructor if needed
        return new PluginClass(definition.config);
    }
}
exports.PluginRegistry = PluginRegistry;
//# sourceMappingURL=pluginRegistry.js.map